package com.xypower.common;

import java.io.File;
import java.io.FileInputStream;
import java.io.FileNotFoundException;
import java.io.FileOutputStream;
import java.io.FilenameFilter;
import java.io.IOException;
import java.util.List;
import java.util.Map;
import java.util.zip.ZipEntry;
import java.util.zip.ZipOutputStream;

public class ZipUtils {

    public static void ZipFolder(File srcDirectory, File zipFile, FilenameFilter filter) {
        ZipOutputStream outZip = null;
        FileInputStream inputStream = null;
        FileOutputStream fileOutputStream = null;

        try {
            fileOutputStream = new FileOutputStream(zipFile);
            outZip = new ZipOutputStream(fileOutputStream);

            int len;
            byte[] buffer = new byte[1024 * 256];
            ZipEntry zipEntry = null;

            File[] subFiles = srcDirectory.listFiles(filter);
            if (subFiles != null && subFiles.length > 0) {
                for (File subFile : subFiles) {
                    zipEntry = new ZipEntry(subFile.getName());
                    outZip.putNextEntry(zipEntry);

                    inputStream = new FileInputStream(subFile);

                    while ((len = inputStream.read(buffer)) != -1) {
                        outZip.write(buffer, 0, len);
                    }
                    FilesUtils.closeFriendly(inputStream);

                    outZip.closeEntry();
                }
            }
        } catch (Exception e) {
            e.printStackTrace();
        } finally {
            FilesUtils.closeFriendly(inputStream);
            try {
                outZip.finish();
            } catch (Exception ex) {
                ex.printStackTrace();
            }
            FilesUtils.closeFriendly(fileOutputStream);
            FilesUtils.closeFriendly(outZip);
        }
    }

    public static void ZipFolders(Map<String, File> srcDirectories, File zipFile, FilenameFilter filter) {
        ZipOutputStream outZip = null;
        FileInputStream inputStream = null;
        FileOutputStream fileOutputStream = null;

        try {
            fileOutputStream = new FileOutputStream(zipFile);
            outZip = new ZipOutputStream(fileOutputStream);
            int len;
            byte[] buffer = new byte[1024 * 256];
            ZipEntry zipEntry = null;

            for (Map.Entry<String, File> srcDirectory : srcDirectories.entrySet()) {
                File[] subFiles = srcDirectory.getValue().listFiles(filter);
                if (subFiles == null || subFiles.length == 0) {
                    continue;
                }
                for (File subFile : subFiles) {
                    zipEntry = new ZipEntry(srcDirectory.getKey() + File.separator + subFile.getName());
                    outZip.putNextEntry(zipEntry);

                    inputStream = new FileInputStream(subFile);

                    while ((len = inputStream.read(buffer)) != -1) {
                        outZip.write(buffer, 0, len);
                    }
                    FilesUtils.closeFriendly(inputStream);

                    outZip.closeEntry();
                }
            }

        } catch (Exception e) {
            e.printStackTrace();
        } finally {
            FilesUtils.closeFriendly(inputStream);
            try {
                outZip.finish();
            } catch (Exception ex) {
                ex.printStackTrace();
            }
            FilesUtils.closeFriendly(fileOutputStream);
            FilesUtils.closeFriendly(outZip);
        }
    }

    public static void ZipFiles(List<String> srcFiles, File zipFile) {

        ZipOutputStream outZip = null;
        FileInputStream inputStream = null;
        FileOutputStream fileOutputStream = null;
        try {
            fileOutputStream = new FileOutputStream(zipFile);
            outZip = new ZipOutputStream(fileOutputStream);
            int len = 0;
            byte[] buffer = new byte[1024 * 256];

            for (String path : srcFiles) {
                File file = new File(path);
                if (!file.exists()) {
                    continue;
                }
                ZipEntry zipEntry = new ZipEntry(srcFiles.size() > 1 ? path.substring(1) : file.getName());
                inputStream = new FileInputStream(file);

                outZip.putNextEntry(zipEntry);
                while ((len = inputStream.read(buffer)) != -1) {
                    outZip.write(buffer, 0, len);
                }
                outZip.closeEntry();
                FilesUtils.closeFriendly(inputStream);
            }

        } catch (Exception e) {
            e.printStackTrace();
        } finally {
            FilesUtils.closeFriendly(inputStream);
            if (outZip != null) {
                try {
                    outZip.finish();
                } catch (Exception ex) {
                    ex.printStackTrace();
                }
            }
            FilesUtils.closeFriendly(fileOutputStream);
            FilesUtils.closeFriendly(outZip);
        }
    }
}
